@@ -1,5 +1,12 @@
+#![deny(missing_docs)]
 #![allow(unknown_lints)]
 
+//! A map-esque data structure that small integer keys for you on insertion.
+//! Key of removed entries are reused for new insertions.
+//! Underlying data is stored in a vector, keys are just indexes of that vector.
+//! The main trick is keeping in-place linked list of freed indexes for reuse.
+
+
 #[cfg(test)]
 mod test;
 
@@ -20,6 +27,23 @@ enum Entry<V> {
     Occupied(V)
 }
 
+/// A map that chooses small integer keys for you.
+/// You store something into this map and then access it by ID returned by it.
+/// For small V entries are expected to take 16 bytes.
+///
+/// Example:
+///
+/// ```
+/// use compactmap::CompactMap;
+///
+/// let mut mymap : CompactMap<String> = CompactMap::new();
+/// let id_qwerty = mymap.insert("qwerty".to_string());
+/// let id_qwertz = mymap.insert("qwertz".to_string());
+/// assert_eq!(mymap[id_qwerty], "qwerty");
+/// for (id, val) in mymap {
+///     println!("{}:{}", id, val);
+/// }
+/// ```
 #[derive(Clone)]
 pub struct CompactMap<V> {
     data: Vec<Entry<V>>,
@@ -27,13 +51,88 @@ pub struct CompactMap<V> {
 }
 
 impl<V> CompactMap<V> {
+    /// Creates an empty `CompactMap`.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use compactmap::CompactMap;
+    /// let mut map: CompactMap<String> = CompactMap::new();
+    /// ```
     pub fn new() -> CompactMap<V> {
         CompactMap {
             data: vec![],
             free_head: usize::MAX
         }
     }
+
+    /// Creates an empty `CompactMap` with space for at least `capacity`
+    /// elements before resizing.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use compactmap::CompactMap;
+    /// let mut map: CompactMap<String> = CompactMap::with_capacity(10);
+    /// ```
+    pub fn with_capacity(capacity: usize) -> Self {
+        CompactMap {
+            data: Vec::with_capacity(capacity),
+            free_head: usize::MAX
+        }
+    }
+
+    /// Returns capacity of the underlying vector.
+    #[inline]
+    pub fn capacity(&self) -> usize {
+        self.data.capacity()
+    }
+
+    /// Reserves capacity for `CompactMap`'s underlying vector.
+    /// If you just cleared M elements from the map and want to insert N
+    /// more elements, you'll probably need to reserve N-M elements.
+    pub fn reserve(&mut self, len: usize) {
+        self.data.reserve(len);
+    }
     
+    /// Reserves capacity for `CompactMap`'s underlying vector.
+    /// If you just cleared M elements from the map and want to insert N
+    /// more elements, you'll probably need to reserve N-M elements.
+    pub fn reserve_exact(&mut self, len: usize) {
+        self.data.reserve_exact(len);
+    }
+    
+    // TODO: keys
+    // TODO: values
+    // TODO: values_mut
+    // TODO: append
+    // TODO: clear
+    // TODO: entry
+    
+    /// Iterating the map to check if it is empty.
+    /// O(n) where n is historical maximum element count.
+    pub fn is_empty_slow(&self) -> bool {
+        return self.len_slow() == 0
+    }
+    
+    /// Inserts a value into the map. The map generates and returns ID of
+    /// the inserted element.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use compactmap::CompactMap;
+    ///
+    /// let mut map = CompactMap::new();
+    /// assert_eq!(map.is_empty_slow(), true);
+    /// assert_eq!(map.insert(37), 0);
+    /// assert_eq!(map.is_empty_slow(), false);
+    ///
+    /// assert_eq!(map.insert(37), 1);
+    /// assert_eq!(map.insert(37), 2);
+    /// assert_eq!(map.insert(44), 3);
+    /// assert_eq!(map.len_slow(), 4);
+    /// ```
     pub fn insert(&mut self, v: V) -> usize {
         let head = self.free_head;
         let entry = Entry::Occupied(v);
@@ -51,7 +150,17 @@ impl<V> CompactMap<V> {
         }
     }
     
-    pub fn take(&mut self, i: usize) -> Option<V> {
+    /// Removes a key from the map, returning the value at the key if the key
+    /// was previously in the map.
+    /// ```
+    /// use compactmap::CompactMap;
+    ///
+    /// let mut map = CompactMap::new();
+    /// let id = map.insert("a");
+    /// assert_eq!(map.remove(id), Some("a"));
+    /// assert_eq!(map.remove(123), None);
+    /// ```
+    pub fn remove(&mut self, i: usize) -> Option<V> {
         if i >= self.data.len() {
             return None
         }
@@ -71,11 +180,7 @@ impl<V> CompactMap<V> {
         } else { unreachable!(); }
     }
     
-    #[inline]
-    pub fn remove(&mut self, i: usize) {
-        self.take(i);
-    }
-    
+    /// Returns a reference to the value corresponding to the key.
     pub fn get(&self, i: usize) -> Option<&V> {
         self.data.get(i).and_then(|entry| match *entry {
             Entry::Empty(_) => None,
@@ -83,6 +188,7 @@ impl<V> CompactMap<V> {
         })
     }
     
+    /// Returns a mutable reference to the value corresponding to the key.
     pub fn get_mut(&mut self, i: usize) -> Option<&mut V> {
         self.data.get_mut(i).and_then(|entry| match *entry {
             Entry::Empty(_) => None,
@@ -90,18 +196,44 @@ impl<V> CompactMap<V> {
         })
     }
     
-    pub fn iter<'a>(&'a self) -> ReadOnlyIter<'a, V> {
-        ReadOnlyIter { iter: self.data.iter(), counter: 0 }
+    /// Returns an iterator visiting all key-value pairs in unspecified order.
+    /// The iterator's element type is `(usize, &'r V)`.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use compactmap::CompactMap;
+    ///
+    /// let mut map = CompactMap::new();
+    /// map.insert("a");
+    /// map.insert("c");
+    /// map.insert("b");
+    ///
+    /// // Print `1: a`, `2: b` and `3: c`.
+    /// for (key, value) in map.iter() {
+    ///     println!("{}: {}", key, value);
+    /// }
+    /// ```
+    pub fn iter<'a>(&'a self) -> Iter<'a, V> {
+        Iter { iter: self.data.iter(), counter: 0 }
     }
     
-    pub fn iter_mut<'a>(&'a mut self) -> MutableIter<'a, V> {
-        MutableIter { iter: self.data.iter_mut(), counter: 0 }
+    /// Returns an iterator visiting all key-value pairs in unspecified order,
+    /// with mutable references to the values.
+    /// The iterator's element type is `(usize, &'r mut V)`
+    pub fn iter_mut<'a>(&'a mut self) -> IterMut<'a, V> {
+        IterMut { iter: self.data.iter_mut(), counter: 0 }
     }
     
+    /// Returns an iterator visiting all key-value pairs in unspecified order,
+    /// the keys, consuming the original `CompactMap`.
+    /// The iterator's element type is `(usize, V)`.
     pub fn into_iter(self) -> IntoIter<V> {
         IntoIter { iter: self.data.into_iter(), counter: 0 }
     }
     
+    /// Iterates the map to get number of elements.
+    /// O(n) where n is historical maximum element count.
     pub fn len_slow(&self) -> usize {
         self.iter().count()
     }
@@ -125,6 +257,13 @@ impl<V> Hash for CompactMap<V> where V: Hash {
     }
 }
 
+// [Partial]Eq impls are based on onces from VecMap
+
+impl<V: PartialEq> PartialEq for CompactMap<V> {
+    fn eq(&self, other: &Self) -> bool {
+        self.iter().eq(other.iter())
+    }
+}
 
 macro_rules! iterate_for_ord_and_eq {
     ($self_:ident, $other:expr, $greater:expr, $less:expr, $j:ident, $k:ident, both_found $code:block) => {
@@ -155,22 +294,7 @@ macro_rules! iterate_for_ord_and_eq {
     }
 }
 
-// Compare for equality disregarting removed values linked list bookkeeping
-impl<V> PartialEq<CompactMap<V>> for CompactMap<V> where V: PartialEq<V> {
-    fn eq(&self, other: &CompactMap<V>) -> bool {
-        iterate_for_ord_and_eq!(self, other, 
-                                false, false, 
-                                j, k, 
-            both_found {
-                if k.ne(j) {
-                    return false
-                }
-            });
-        true
-    }
-}
-
-impl<V> Eq for CompactMap<V> where V: Eq { }
+impl<V: Eq> Eq for CompactMap<V> {}
 
 // We are greater then them iif { { we have i'th slot 
 // filled in and they don't } or { data in i'th slot compares
@@ -299,11 +423,12 @@ macro_rules! generate_iterator {
     };
 }
 
-pub struct ReadOnlyIter<'a, V : 'a> {
+/// An iterator over the key-value pairs of a map.
+pub struct Iter<'a, V : 'a> {
     iter: slice::Iter<'a, Entry<V>>,
     counter : usize,
 }
-impl<'a,V> Iterator for ReadOnlyIter<'a,V> {
+impl<'a,V> Iterator for Iter<'a,V> {
     type Item = (usize, &'a V);
     
     #[allow(match_ref_pats)]
@@ -313,18 +438,19 @@ impl<'a,V> Iterator for ReadOnlyIter<'a,V> {
 }
 impl<'a,V> IntoIterator for &'a CompactMap<V> {
     type Item = (usize, &'a V);
-    type IntoIter = ReadOnlyIter<'a, V>;
-    fn into_iter(self) -> ReadOnlyIter<'a, V> {
-        ReadOnlyIter { iter: self.data.iter(), counter: 0 }
+    type IntoIter = Iter<'a, V>;
+    fn into_iter(self) -> Iter<'a, V> {
+        Iter { iter: self.data.iter(), counter: 0 }
     }
 }
 
-
-pub struct MutableIter<'a, V : 'a> {
+/// An iterator over the key-value pairs of a map, with the
+/// values being mutable.
+pub struct IterMut<'a, V : 'a> {
     iter: slice::IterMut<'a, Entry<V>>,
     counter : usize,
 }
-impl<'a,V:'a> Iterator for MutableIter<'a,V> {
+impl<'a,V:'a> Iterator for IterMut<'a,V> {
     type Item = (usize, &'a mut V);
     
     #[allow(unused_lifetimes,match_ref_pats)]
@@ -335,13 +461,13 @@ impl<'a,V:'a> Iterator for MutableIter<'a,V> {
 
 impl<'a,V:'a> IntoIterator for &'a mut CompactMap<V> {
     type Item = (usize, &'a mut V);
-    type IntoIter = MutableIter<'a, V>;
-    fn into_iter(self) -> MutableIter<'a, V> {
-        MutableIter { iter: self.data.iter_mut(), counter: 0 }
+    type IntoIter = IterMut<'a, V>;
+    fn into_iter(self) -> IterMut<'a, V> {
+        IterMut { iter: self.data.iter_mut(), counter: 0 }
     }
 }
 
-
+/// A consuming iterator over the key-value pairs of a map.
 pub struct IntoIter<V> {
     iter: vec::IntoIter<Entry<V>>,
     counter : usize,
